<?php
/**
 * LuhnNumberValidator class file.
 *
 * @author Mario De Weerd
 * @copyright 2011-2013 Ynamics SARL
 */
/**
 * Copyright © 2012-2013 by Ynamics, SARL
 * All rights reserved.
 *
 * [BSD License]
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name of Ynamics nor the names of its contributors may
 *   be used to endorse or promote products derived from this software without
 *   specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/**
 * LuhnNumberValidator class file.
 *
 * Used to validate that a string corresponds to a valid Luhn number (used in mobile phones).
 */
class LuhnNumberValidator extends CValidator
{
    /**
     * Length of the Luhn number to validate
     * @var integer.
     */
    public $length;
    /**
     * Pattern of the Luhn number.
     *
     * @var unknown_type
     */
    public $pattern;
    /**
     * Type of luhn number to validate.
     *
     * The default pattern and length is set based on the 'type'.
     * The setting is optional.
     *
     * Known types:
     *  'SIRET' - French SIREN number
     *  'SIREN' - French SIRET number.
     *  'ICCID' - ICCID (example: sim card)
     *  'IMEI'  - IMEI number (Mobile phone)
     * @var string
     */
    public $type;

    /**
     * If an empty value is allowed.
     *
     * @var unknown_type
     */
    public $allowEmpty=false;
    /**
     * (non-PHPdoc)
     * @see CValidator::validateAttribute()
     */
    protected function validateAttribute($object,$attribute)
    {
        $value=$object->$attribute;
        if($this->allowEmpty && $this->isEmpty($value)) {
            return;
        }
        if(!$this->validateValue($value)) {
            $params=array('{attribute}'=>$object->$attribute);
            $message=$this->message!==null?Yii::t('app',$this->message,$params):Yii::t('app','"{attribute}" is not a valid Luhn number.',$params);
            $this->addError($object,$attribute,$message);
        }
    }

    /**
     * Get the pattern that is used to validate the number.
     *
     * Uses the 'type' parameter for the default values.
     *
     * @throws InvalidArgumentException
     * @return string
     */
    public function getPattern() {
        if(!isset($this->pattern)) {
            if (isset($this->type)) {
                switch ($this->type) {
                    case 'SIRET':
                        /* French SIRET number */
                        $length=14;
                        break;
                    case 'SIREN':
                        /* French SIREN number */
                        $length=9;
                        break;
                    case 'IMEI':
                        /* French IMEI number */
                        $length=15;
                        break;
                    case 'ICCID':
                        /*
                         * The format of the ICCID is: MMCC IINN NNNN NNNN NN C x
                         *
                         * MM = Constant (ISO 7812 Major Industry Identifier, = 89 for "Telecommunications administrations and private operating agencies")
                         * CC = Country Code (i.e. 61 = Australia, 86 = China)
                         * II = Issuer Identifier (AAPT = 14, EZI-PhoneCard = 88, Hutchison = 06, Optus = 02/12/21/23, Telstra = 01, Telstra Business = 00/61/62, Vodafone = 03)
                         * N{12} = Account ID ("SIM number")
                         * C = Checksum calculated from the other 19 digits using the Luhn algorithm.
                         * x = An extra 20th digit is returned by the 'AT!ICCID?' command,
                         *     but doesn't seem to be an official part of the ICCID.
                         */
                        $length=19;
                        break;
                }
            }
            if (isset($this->length)) {
                if (isset($length)&&($length!=$this->length)) {
                    throw new InvalidArgumentException("Requested length {$this->length} incompatible with $this->type");

                }
            } elseif(isset($length)) {
                $this->length=$length;
            }
            if(isset($this->length)) {
                $this->pattern='/^\d{'."$this->length,$this->length".'}$/';
            } else {
                $this->pattern='/^\d+$/';
            }
        }
        return $this->pattern;
    }
    /**
     * Validates a static value to see if it is a Luhn number or not
     * Note that this method does not respect {@link allowEmpty} property.
     * @param mixed $value the value to be validated
     * @return boolean whether the value is a valid luhn number
     * @since 1.1.1
     */
    public function validateValue($Luhn)
    {
        $result=true;  // Suppose result is ok until something fails.
        $result=$result&&(is_string($Luhn)&&(preg_match($this->getPattern(),$Luhn)!==0));

        if($result) {
            $result=$this->validateLuhn($Luhn);
        }

        if($result && $this->type==='SIRET' && strlen($Luhn)>9) {
            // The French siret also holds the SIREN number which is also Luhn validated.
            $result=$this->validateLuhn(substr($Luhn,0,9));
        }
        return $result;
    }

    private function validateLuhn($value) {
        // Validate the number.
        // - Double every other digit.
        // - Compute the sum
        $sum=0;
        $even=false;
        for($i=strlen($value)-1; $i>=0; $i--) {
            $num = intval($value[$i]);
            if($even) {
                $num*=2;
                $num= ($num%10) + intval($num/10);
            }
            $even=!$even;
            $sum+=$num;
        }
        return ( ($sum%10)==0 );
    }

    /**
     * Returns the JavaScript needed for performing client-side validation.
     * @param CModel $object the data object being validated
     * @param string $attribute the name of the attribute to be validated.
     * @return string the client-side validation script.
     * @see CActiveForm::enableClientValidation
     */
    public function clientValidateAttribute($object,$attribute)
    {

    }
}
